# Generated by Django 3.2.4 on 2022-03-22 14:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ashley", "0005_uploadimage"),
    ]

    operations = [
        migrations.AddField(
            model_name="lticontext",
            name="is_marked_locked",
            field=models.BooleanField(default=False),
        ),
    ]
