# Generated by Django 3.2.4 on 2021-07-13 13:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("forum", "0015_auto_20210217_1351"),
    ]

    operations = [
        migrations.AddField(
            model_name="forum",
            name="archived",
            field=models.BooleanField(default=False),
        ),
    ]
